/*
 * Small sample applications.
 * There are no comments, because it is really simple to use the
 * 1wire functions...
 */

#ifdef ETHERNUT2
#include <dev/lanc111.h>
#else
#include <dev/nicrtl.h>
#endif

#include <stdio.h>
#include <dev/debug.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/heap.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/confnet.h>
#include <arpa/inet.h>


#include "1wire.h"

typedef struct _DS1W_DEVICE DS1W_DEVICE;
struct _DS1W_DEVICE
{
    u_char ROM[8];
    float temp;
    DS1W_DEVICE *next;
};

DS1W_DEVICE * ds1w_devices = 0;
    
THREAD (ds1w, arg)
{
    u_char buf[8];
    DS1W_DEVICE *dev;
    int r;
    
    do
    {
        r = DS1WSearchRom (buf);
        if (r >= 0)
        {
            dev = (DS1W_DEVICE *) NutHeapAllocClear (sizeof (*dev));
            memcpy (dev->ROM, buf, 8);
            dev->next = ds1w_devices;
            ds1w_devices = dev;
        }
    } 
    while (r > 0);
    
    for (;;)
    {
        DS1WInit();
        DS1WWriteByte(DS1W_SKIP_ROM);
        DS1WWriteByte(DS1W_CONVERT_T);		/* Start temperature conversion for each device */
        NutSleep (1000);			/* Wait until conversion is finished */
        
        dev = ds1w_devices;
        
        for (; dev; dev = dev->next)		/* Read temperatur from each device */
        {
            DS1WInit();
            DS1WWriteByte(DS1W_MATCH_ROM);
            DS1WWriteBuf (dev->ROM, 8);
            DS1WWriteByte(DS1W_READ_SCRATCHPAD);
            DS1WReadBuf (buf, 7);
            dev->temp = ((16.0 - buf[6]) / 16.0) - 0.25 + (buf[0] >> 1);
        }
    }
}    
    
/* simple telnet server */
THREAD (telnet, arg)
{
    TCPSOCKET *sock;
    FILE *stream;
    DS1W_DEVICE *dev;
    u_char id = (u_char) ((u_short) arg);
    
    for(;;)
    {
        if ((sock = NutTcpCreateSocket()) == 0)
            puts ("Error creating socket");
            
        NutTcpAccept (sock, 23);
        printf ("[%d] Connected\n", id);
        stream = _fdopen((int) sock, "r+");
        
        if (!ds1w_devices)
            fputs ("no ds1w device found.", stream);
        else
        {
            for (;;)
            {
                dev = ds1w_devices;
                for (; dev; dev = dev->next)
                {
                    fprintf (stream, "%02X%02X%02X%02X%02X%02X%02X%02X: ", dev->ROM[0], dev->ROM[1], 
                    dev->ROM[2], dev->ROM[3], dev->ROM[4], dev->ROM[5], dev->ROM[6], dev->ROM[7]);
                    fprintf (stream, "%f, ", dev->temp);
                }
                fputs (";\r\n", stream);
                fflush (stream);
                
                if (sock->so_state != TCPS_ESTABLISHED)
                    break;
                NutSleep (2000);
            }
        }
        fclose (stream);
        NutTcpCloseSocket (sock);
        printf ("[%d] Disonnected\n", id);
    }
}
    
void main(void)
{
    int i;
    char tname[] = "telnet0";
    
    NutRegisterDevice(&devDebug0, 0, 0);
    freopen("uart0", "w", stdout);
    puts ("Starting temperature server..\n");

    if (NutRegisterDevice(&DEV_ETHER, 0x8300, 5))
        puts("Registering device failed");
    NutNetAutoConfig ("eth0");
    printf("%s ready\n", inet_ntoa(confnet.cdn_ip_addr));

    NutThreadCreate ("ds1w", ds1w, 0, 512);
    
    for (i=0; i < 5; i++)
    {
        tname[6] = i + '0';
        NutThreadCreate (tname, telnet, (void *) (u_short) i, 256);
    }
    
    for (;;)
    {
    	printf ("Free heap: %d bytes\n", NutHeapAvailable());
        NutSleep (3000);
    }
        
}
